from collections import namedtuple
import datetime
import nose.tools as nt
from mock import patch
from ckan.common import config
import ckan.logic.action.create as create


from ckanext.odsh.helpers_tpsh import (
    map_dct_type_to_ckan_type,
    map_ckan_type_to_dct_type,
    add_pkg_to_collection,
    correct_missing_relationship,
    get_language_of_package
)


class TestMatchDctTypeToCkanType(object):
    
    def test_it_returns_collection(self):
        dct_type = 'http://dcat-ap.de/def/datasetTypes/collection'
        ckan_type = map_dct_type_to_ckan_type(dct_type)
        expected_ckan_type = 'collection'
        nt.assert_equal(ckan_type, expected_ckan_type)
    
    def test_it_returns_none_for_unknown_type(self):
        nt.assert_equal(
            map_dct_type_to_ckan_type('some unknown type'), 
            None
        )


class TestMatchCkanTypeToDctType(object):
    
    def test_it_returns_url_for_collection(self):
        ckan_type = 'collection'
        dct_type = map_ckan_type_to_dct_type(ckan_type)
        expected_dct_type = 'http://dcat-ap.de/def/datasetTypes/collection'
        nt.assert_equal(
            dct_type, expected_dct_type
        )


FakePackageRelationship = namedtuple(
    'FakePackageRelationship',
    '''
    object_package_id
    revision_id
    subject_package_id
    id
    type
    '''
)

class Test_correct_missing_relationship(object):
    def setUp(self):
        self.relationships_from_model = [
            FakePackageRelationship(
                object_package_id=u'object package id',
                revision_id=u'revision id',
                subject_package_id=u'subject package id',
                id=u'id',
                type=u'type'
            )
        ]

    def test_it_does_not_modify_pkg_dict_if_no_model_relationships(self):
        relationships_from_model = []
        original_pkg_dict = {'id': 'some_id'}
        pkg_dict = dict(original_pkg_dict)
        correct_missing_relationship(
            pkg_dict, relationships_from_model
        )
        nt.assert_equal(pkg_dict, original_pkg_dict)
    
    def test_it_does_not_modify_pkg_dict_if_relationships_already_in_dict(self):
        original_pkg_dict = {
            u'type': u'dataset',
            u'relationships_as_subject': [
                {
                    u'__extras': {
                        u'object_package_id': u'original object package id', 
                        u'revision_id': u'original revision id', 
                        u'subject_package_id': u'original subject package id'
                    }, 
                    u'comment': u'', 
                    u'id': u'original id', 
                    u'type': u'original type'
                }
            ]
        }
        pkg_dict = dict(original_pkg_dict)
        correct_missing_relationship(
            pkg_dict, self.relationships_from_model
        )
        nt.assert_equal(pkg_dict, original_pkg_dict)
    
    def test_it_does_not_modify_pkg_dict_if_type_collection(self):
        original_pkg_dict = {
            u'type': u'collection',
            u'relationships_as_subject': [
                {
                    u'__extras': {
                        u'object_package_id': u'original object package id', 
                        u'revision_id': u'original revision id', 
                        u'subject_package_id': u'original subject package id'
                    }, 
                    u'comment': u'', 
                    u'id': u'original id', 
                    u'type': u'original type'
                }
            ]
        }
        pkg_dict = dict(original_pkg_dict)
        correct_missing_relationship(
            pkg_dict, self.relationships_from_model
        )
        nt.assert_equal(pkg_dict, original_pkg_dict)
    
    def test_it_adds_relationships_if_not_already_in_dict(self):
        pkg_dict = {
            u'type': u'dataset',
            u'relationships_as_subject': []
        }
        correct_missing_relationship(
            pkg_dict, self.relationships_from_model
        )
        expected_pkg_dict = {
            u'type': u'dataset',
            u'relationships_as_subject': [
                {
                    u'__extras': {
                        u'object_package_id': u'object package id', 
                        u'revision_id': u'revision id', 
                        u'subject_package_id': u'subject package id'
                    }, 
                    u'comment': u'', 
                    u'id': u'id', 
                    u'type': u'type'
                }
            ]
        }

        from_relationships = lambda d, key: d.get('relationships_as_subject')[0].get(key)
        from_extras = lambda d, key: d.get('relationships_as_subject')[0].get('__extras').get(key)
        
        # assert
        nt.assert_equal(pkg_dict.get('type'), 'dataset')
        
        for key in ('id', 'type'):
            nt.assert_true(from_relationships(pkg_dict, key) is not None)
            nt.assert_equal(
                from_relationships(pkg_dict, key), 
                from_relationships(expected_pkg_dict, key)
            )

        for key in ('object_package_id', 'revision_id', 'subject_package_id'):
            nt.assert_true(
                pkg_dict is not None)
            nt.assert_equal(
                from_extras(pkg_dict, key),
                from_extras(expected_pkg_dict, key)
            )

    
class Test_get_language_of_package(object):
    def setUp(self):
        config.update({'ckanext.odsh.language_mapping': '/usr/lib/ckan/default/src/ckanext-odsh/language_mapping.json'})
    
    def tearDown(self):
        config.clear()

    def test_it_returns_Englisch(self):
        test_package = {
                'id': u'language_test',
                u'extras': [
                    {u'key': u'language', u'value': u'http://publications.europa.eu/resource/authority/language/ENG'},
                ]
            }
        nt.assert_equal(get_language_of_package(test_package),'Englisch')
    
    def test_it_returns_None_if_language_id_not_in_dict(self):
        test_package = {
                'id': u'language_test',
                u'extras': [
                    {u'key': u'language', u'value': u'tlhIngan Hol'},
                ]
            }
        nt.assert_equal(get_language_of_package(test_package), None)
    
    def test_it_returns_None_if_language_not_in_pkg_dict(self):
        test_package = {}
        nt.assert_equal(get_language_of_package(test_package), None)

